@@ -26,7 +26,7 @@
 ##  shiela.pl: control program (syntax: Perl)
 ##
 
-my $version = '1.1.1';
+my $version = '1.1.2';
 
 require 5.005;
 
@@ -368,7 +368,7 @@ sub parse_config {
 ##  Determine OSSP shiela configuration.
 ##
 ##  We theoretically could directly operate on the syntax tree as
-##  created by parse_config() above. But for convinience reasons and
+##  created by parse_config() above. But for convenience reasons and
 ##  to greatly simplify the processing, we use callback functions for
 ##  parse_config() and build an own configuration structure.
 ##  _________________________________________________________________
@@ -556,7 +556,7 @@ sub pattern_match {
 ##  required by most of the CVS client commands), e.g. when an import
 ##  is done locally (no client/server). So we generally use the CVS
 ##  client/server protocol to communicate with a spawned CVS server
-##  process and act as we would be a regular CVS client. For convinience
+##  process and act as we would be a regular CVS client. For convenience
 ##  reasons, the communication is encapsulated in a "CVS" class object.
 ##  _________________________________________________________________
 ##
@@ -658,7 +658,7 @@ sub recv {
     }
 }
 
-#   convinience wrapper: receive a response
+#   convenience wrapper: receive a response
 sub result {
     my $cvs = shift;
     my $line;
@@ -674,7 +674,7 @@ sub result {
     }
 }
 
-#   convinience wrapper: provide a file entry
+#   convenience wrapper: provide a file entry
 sub entry {
     my $cvs = shift;
     my @files = @_;
@@ -684,7 +684,7 @@ sub entry {
     }
 }
 
-#   convinience wrapper: provide one or more global options
+#   convenience wrapper: provide one or more global options
 sub global_options {
     my $cvs = shift;
     my @opts = @_;
@@ -693,7 +693,7 @@ sub global_options {
     }
 }
 
-#   convinience wrapper: provide one or more arguments
+#   convenience wrapper: provide one or more arguments
 sub arguments {
     my $cvs = shift;
     my @args = @_;
@@ -702,7 +702,7 @@ sub arguments {
     }
 }
 
-#   convinience wrapper: configure a directory
+#   convenience wrapper: configure a directory
 sub directory {
     my $cvs = shift;
     my ($dir) = @_;
@@ -811,7 +811,7 @@ package main;
 ##
 ##  Common file operations.
 ##
-##  This is nothing more than a convinience function for
+##  This is nothing more than a convenience function for
 ##  the common file operations we have do.
 ##  _________________________________________________________________
 ##
@@ -1440,7 +1440,7 @@ sub hook_loginfo_collect {
     $RT->{cvsop} = 'commit-file';
 
     #   handle special invocation under `cvs add <dir>'
-    if ($PA->{ARG}->[1] eq '- New directory') { # see CVS' src/add.c
+    if (defined($PA->{ARG}->[1]) and $PA->{ARG}->[1] eq '- New directory') { # see CVS' src/add.c
         #   Hmmm... we always just deal with files in OSSP shiela, so there
         #   is no obvious and consistent way to deal now with only a
         #   plain directory. And there is also no log message provided
@@ -1456,7 +1456,7 @@ sub hook_loginfo_collect {
     #   CVS only calls us inside the loginfo hook and never in the
     #   commitinfo hook before. Additionally CVS doesn't provide us with
     #   the %{sVvto} information :(
-    if ($PA->{ARG}->[1] eq '- Imported sources') { # see CVS' src/import.c
+    if (defined($PA->{ARG}->[1]) and $PA->{ARG}->[1] eq '- Imported sources') { # see CVS' src/import.c
         #   I = ignored
         #   L = link (=error),
         #   N = new file
@@ -1674,11 +1674,8 @@ sub hook_loginfo_collect {
         }
         else {
             if ($Io eq 'A') {
-                my $io = new IO::File "<$Is" || die "unable to open $Is for reading";
-                my $l = 0;
-                $l++ while (<$io>);
-                $io->close;
-                $Id = sprintf("+%d/-%d", $l, 0);
+                #   determined later below when we have to read in the
+                #   whole content anyway in order to create the difference.
             }
             elsif ($Io eq 'M') {
                 if ($rcslog =~ m|\ndate:.*lines:\s*([\d \t+-]+)|s) {
@@ -1710,10 +1707,14 @@ sub hook_loginfo_collect {
             unlink("$RT->{tmpfile}.all");
             my $io = new IO::File ">$RT->{tmpfile}.all"
                 || die "unable to open temporary file $RT->{tmpfile}.all for writing";
+            my $l = 0;
             if (not $RT->{useserver}) {
                 my $cvs = new IO::File "$RT->{cvs} -f -Q -n update -p -r$Iv '$Is'|"
                     || die "unable to open CVS command pipe for reading";
-                $io->print($_) while (<$cvs>);
+                while (<$cvs>) {
+                    $io->print($_);
+                    $l++;
+                }
                 $cvs->close;
             }
             else {
@@ -1722,9 +1723,14 @@ sub hook_loginfo_collect {
                 $cvs->entry($Is);
                 $cvs->arguments("-p", "-r$Iv", $Is);
                 $cvs->send("update");
-                $io->print(scalar $cvs->result);
+                my $r = scalar $cvs->result;
+                $io->print($r);
                 $cvs->close;
+                if ($r ne '') {
+                    $l++ while ($r =~ m/^/mg);
+                }
             }
+            $Id = sprintf("+%d/-%d", $l, 0) if (not ($Ik eq 'b' or -B $Is));
             $io->close;
 
             if ($Ik eq 'b' or -B $Is) {
